#include <iostream>
#include <fstream>
#include <iomanip>
#include <string>
#include <cctype>
#include "Parser.h"

using namespace std;

void Parser::addToken(PTokenData* t)
{
	addToken((PToken*)t);
}

void Parser::addToken(PToken* t)
{
	t->line = lineno;
	tokens->push_back(t);
}

void Parser::parse() 
{
	string strTemp;
	while(!file.eof())
	{
		char temp = file.peek();
		if(isspace(temp))
		{
			// if whitespace, actually read the character, then restart loop and try again.
			file.get();

			// if the character is a new line increment lines count
			if(temp=='\n')
				lineno++;
			continue;
		}

		if(temp=='>')
		{
			PToken* token = new PToken;
			
			file.get();
			temp = file.peek();
			
			if(temp=='=')
			{
				token->type = GTHANEQUAL;
				file.get();
			} else
				token->type = GTHAN;

			addToken(token);
			continue;
		}

		if(temp=='<')
		{
			PToken* token = new PToken;
			
			file.get();
			temp = file.peek();
			
			if(temp=='=')
			{
				token->type = LTHANEQUAL;
				file.get();
			} else
				token->type = LTHAN;

			addToken(token);
			continue;
		}


		if(temp=='+')
		{
			PToken* token = new PToken;
			file.get();
			temp = file.peek();
			if(temp=='+')
			{
				token->type = PLUSPLUS;	
				file.get();
			} else if(temp=='=') {
				token->type = PLUSEQUALS;
				file.get();
			} else
				token->type = PLUS;
			addToken(token);
			continue;
		}

		if(temp=='-')
		{
			PToken* token = new PToken;
			file.get();
			temp = file.peek();
			if(temp=='-')
			{
				token->type = MINUSMINUS;	
				file.get();
			} else if(temp=='=') {
				token->type = MINUSEQUALS;
				file.get();
			} else
				token->type = MINUS;
			addToken(token);
			continue;
		}

		if(isalpha(temp) || temp=='_')
		{
			//At first, this was simpler, but just doing
			// ifstream >> string; was grabbing other stuff
			// that needs to be it's own token.
			string* s = new string;
			while(isalpha(temp) || isdigit(temp) || temp=='_')
			{
				file.get();
				*s += temp;
				temp = file.peek();
			}
			//cout << "new ident code result <" << *s << ">" << endl;
			PTokenData* token = new PTokenData;
			token->type = IDENT;
			token->data =(char*) s;
			addToken(token);
			//cout << "IDENT: " << (*s) << endl;
			continue;
		}

		if(temp=='/')
		{
			// A forward slash is either division or first of two for a comment
			// We'll toss the slash with a first get statement, then peek to see
			// if it's a comment. If it's a comment we read the rest of the line
			// then toss it. Else we create a FSLASH token.
			file.get();
			temp = file.peek();
			if(temp=='/')
			{
				getline(file, strTemp);
				strTemp.clear();
				continue;
			}
			// if it's not a comment, then it's division, 
			// create the token and add it
			PToken* token = new PToken;
			token->type = FSLASH;
			addToken(token);
			continue;
		}

		if(temp=='0')
		{
			PTokenData* token = new PTokenData;
			token->type = INTEGER;
			int a;

			file.get();
			temp = file.peek();
			file.unget();
				
			if(temp=='x'||temp=='X')
				file >> hex >> a;
			else
				file >> dec >> a;
			
			token->data =(char*) a;
			addToken(token);
			continue;
		}

		if(isdigit(temp))
		{
			PTokenData* token = new PTokenData;
			token->type = INTEGER;
			int a;

			file >> dec >> a;
			token->data =(char*) a;

			addToken(token);
			continue;
		}

		PToken* token = new PToken;
		token->type = 0;
		switch(temp)
		{
		case '=':
			token->type = EQUALS;
			break;
		case '*':
			token->type = STAR;
			break;
		case ';':
			token->type = SEMI;
			break;
		case '.':
			token->type = PERIOD;
			break;
		case '{':
			token->type = LBRACE;
			break;
		case '}':
			token->type = RBRACE;
			break;
		case '[':
			token->type = LBRAC;
			break;
		case ']':
			token->type = RBRAC;
			break;
		case '(':
			token->type = LPAREN;
			break;
		case ')':
			token->type = RPAREN;
			break;
		case ':':
			token->type = COLON;
			break;
		case ',':
			token->type = COMMA;
			break;
		}

		file.get();
		if(token->type == 0)
		{
			delete token;
		} else {
			addToken(token);
		}
	}
}

Parser::Parser(string filename) : file(filename.c_str())
{
	if(!file.is_open())
	{
		cerr << "BPL: " << filename << ": Error opening file" << endl;
		tokens = NULL;
		return;
	}
	tokens = new list<PToken*>();
	lineno = 1;
	parse();
}

Parser::~Parser()
{
	if(file.is_open())
		file.close();
	if(tokens != NULL)
		delete tokens;
}
